@@ -25,6 +25,9 @@
 ##' @param di_80_index_cutoff Threshold used for determining disproportionate impact using the 80\% index; passed to \link[DisImpact]{di_80_index}; defaults to 0.80.
 ##' @param di_80_index_reference_groups A character vector of the same length as \code{group_vars} that indicates the reference group value for each group variable in \code{group_vars} when determining disproportionate impact using the 80\% index; defaults to \code{'hpg'} (highest performing group as reference), but could also be \code{'overall'} or \code{'all but current'}.
 ##' @param check_valid_reference Check whether \code{ppg_reference_groups} and \code{di_80_index_reference_groups} contain valid values; defaults to \code{TRUE}.
+##' @param parallel If \code{TRUE}, then perform calculations in parallel based on the scenarios specified by \code{scenario_repeat_by_vars}.  Defaults to \code{FALSE}.  Parallel execution is based on the \code{parallel} package included in base R, using \link[parallel]{parLapply} on Windows and \link[parallel]{mclapply} on POSIX-based systems (Linux/Mac).
+##' @param parallel_n_cores The number of CPU cores to use if \code{parallel=TRUE}.  Defaults to the maximum number on the system.
+##' @param parallel_split_to_disk If \code{TRUE} and \code{parallel=TRUE}, then create intermediate data sets for each scenario generated by \code{scenario_repeat_by_vars}, write them to disk, and import the required data set when necessary for each scenario executing in parallel.  This feature is useful when the data set specified by \code{data} is very large and parallel execution is desired for speed in order to reduce the likelihood of consuming all the system's memory and crashing.  Note that there is an overhead I/O cost on speed when this feature is used.  Defaults to \code{FALSE}.
 ##' @return A summarized data set (data frame) consisting of:
 ##' \itemize{
 ##'   \item \code{success_variable} (elements of \code{success_vars}),
@@ -43,11 +46,13 @@
 ##'   , group_vars=c('Ethnicity', 'Gender'), cohort_vars=c('Cohort')
 ##'   , ppg_reference_groups='overall')
 ##' @import dplyr
+##' @import parallel
+##' @import fst
 ##' @importFrom tidyselect everything one_of
 ##' @importFrom purrr pmap
 ##' @importFrom tidyr unnest
 ##' @export
-di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenario_repeat_by_vars=NULL, exclude_scenario_df=NULL, weight_var=NULL, include_non_disagg_results=TRUE, ppg_reference_groups='overall', min_moe=0.03, use_prop_in_moe=FALSE, prop_sub_0=0.5, prop_sub_1=0.5, di_prop_index_cutoff=0.80, di_80_index_cutoff=0.80, di_80_index_reference_groups='hpg', check_valid_reference=TRUE) {
+di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenario_repeat_by_vars=NULL, exclude_scenario_df=NULL, weight_var=NULL, include_non_disagg_results=TRUE, ppg_reference_groups='overall', min_moe=0.03, use_prop_in_moe=FALSE, prop_sub_0=0.5, prop_sub_1=0.5, di_prop_index_cutoff=0.80, di_80_index_cutoff=0.80, di_80_index_reference_groups='hpg', check_valid_reference=TRUE, parallel=FALSE, parallel_n_cores=parallel::detectCores(), parallel_split_to_disk=FALSE) {
   stopifnot(length(group_vars) == length(ppg_reference_groups) | length(ppg_reference_groups) == 1)
   stopifnot(length(group_vars) == length(di_80_index_reference_groups) | length(di_80_index_reference_groups) == 1)
 
@@ -78,6 +83,11 @@ di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenari
       }
     }
   }
+
+  # Check parallel parameters
+  if (isTRUE(parallel_split_to_disk) & isFALSE(parallel)) {
+    stop('`parallel_split_to_disk=TRUE` only works if `parallel=TRUE`.')
+  }
   
   # Add a variable for non-disaggregated results
   if (include_non_disagg_results) {
@@ -194,7 +204,7 @@ di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenari
                                    slice(i) %>% 
                                    select(one_of(vars_specific)) %>%
                                    left_join(data %>% mutate(row_index=row_number())) %>%
-
+                                   suppressMessages %>% # Source of many "Joining, by", so just remove them
                                    filter(!is.na(row_index)) %>% # No match
                                    group_by_at(vars(one_of(vars_specific))) %>%
                                    summarize(want_indices=list(row_index), n_rows=n()) %>%
@@ -217,6 +227,14 @@ di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenari
       bind_rows %>%
       filter(n_rows > 0) %>%
       select(-n_rows)
+
+    dRepeatScenarios$parallel_split_to_disk <- parallel_split_to_disk
+    dRepeatScenarios$split_filename <- tempfile(rep('file', nrow(dRepeatScenarios)), fileext='.fst')
+    if (parallel & parallel_split_to_disk) {
+      # Write out a file for each repeat scenario
+      message(paste0('NOTE: Since `parallel_split_to_disk=TRUE`, writing out ', nrow(dRepeatScenarios), ' intermediate data sets for use in parallel execution.'))
+      lapply(1:nrow(dRepeatScenarios), FUN=function(i) write_fst(x=data[dRepeatScenarios %>% slice(i) %>% select(want_indices) %>% unlist, ], dRepeatScenarios$split_filename[i]))
+    }
   }
   
   ## if (!is.null(scenario_repeat_by_vars)) {
@@ -271,9 +289,13 @@ di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenari
     select(success_var, group_var, cohort_var, ppg_reference_group, min_moe, use_prop_in_moe, prop_sub_0, prop_sub_1, ppg_check_valid_reference, di_prop_index_cutoff, di_80_index_cutoff, di_80_index_reference_group, di_80_check_valid_reference)
 
   # Function to iterate for each scenario
-  iterate <- function(success_var, group_var, cohort_var, ppg_reference_group, min_moe, use_prop_in_moe, prop_sub_0, prop_sub_1, ppg_check_valid_reference, di_prop_index_cutoff, di_80_index_cutoff, di_80_index_reference_group, di_80_check_valid_reference, subset_idx) {
-    
-    data <- data[subset_idx, ]
+  iterate <- function(success_var, group_var, cohort_var, ppg_reference_group, min_moe, use_prop_in_moe, prop_sub_0, prop_sub_1, ppg_check_valid_reference, di_prop_index_cutoff, di_80_index_cutoff, di_80_index_reference_group, di_80_check_valid_reference, subset_idx, parallel_split_to_disk, split_filename) {
+
+    if (!parallel_split_to_disk) {
+      data <- data[subset_idx, ]
+    } else {
+      data <- read_fst(path=split_filename)
+    }
 
     # Remove rows with missing value for success_var
     data <- data[!is.na(data[[success_var]]), ]
@@ -320,23 +342,72 @@ di_iterate <- function(data, success_vars, group_vars, cohort_vars=NULL, scenari
   # Iterate for all scenarios
   if (is.null(scenario_repeat_by_vars)) {
     subset_idx <- 1:nrow(data)
-    pmap(dScenarios %>% mutate(subset_idx=list(subset_idx)), iterate) %>%
+    pmap(dScenarios %>% mutate(subset_idx=list(subset_idx), parallel_split_to_disk=FALSE, split_filename='NOT_USED'), iterate) %>%
       bind_rows
   } else {
-    dRepeatScenarios$df_results <- lapply(1:nrow(dRepeatScenarios)
-                                     , FUN=function(i) {
-                                       # data <- data %>% slice(dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist)
-                                       # subset_idx <- dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist
-                                       subset_idx <- dRepeatScenarios %>% slice(i) %>% select(want_indices) %>% unlist
-                                       pmap(dScenarios %>% mutate(subset_idx=list(subset_idx)), iterate) %>%
-                                         bind_rows
-                                     }
-                                     )
+    if (!parallel) {
+      dRepeatScenarios$df_results <- lapply(1:nrow(dRepeatScenarios)
+                                       , FUN=function(i) {
+                                         # data <- data %>% slice(dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist)
+                                         # subset_idx <- dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist
+                                         subset_idx <- dRepeatScenarios %>% slice(i) %>% select(want_indices) %>% unlist
+                                         pmap(dScenarios %>% mutate(subset_idx=list(subset_idx), parallel_split_to_disk=parallel_split_to_disk, split_filename=dRepeatScenarios$split_filename[i]), iterate) %>%
+                                           bind_rows
+                                       }
+                                       )
+    } else if (parallel & Sys.info()[['sysname']] != 'Windows'){
+      
+      message(paste0('NOTE: Since `parallel=TRUE`, will attempt to use ', parallel_n_cores, ' CPU cores to execute ', nrow(dRepeatScenarios), ' scenarios in parallel.  The user could change the number of parallel cores with the `parallel_n_cores` argument.'))
+
+      dRepeatScenarios$df_results <- mclapply(1:nrow(dRepeatScenarios)
+                                       , FUN=function(i) {
+                                         # data <- data %>% slice(dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist)
+                                         # subset_idx <- dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist
+                                         subset_idx <- dRepeatScenarios %>% slice(i) %>% select(want_indices) %>% unlist
+                                         pmap(dScenarios %>% mutate(subset_idx=list(subset_idx), parallel_split_to_disk=parallel_split_to_disk, split_filename=dRepeatScenarios$split_filename[i]), iterate) %>%
+                                           bind_rows
+                                       }
+                                       , mc.cores=parallel_n_cores
+                                       )
+      if (parallel_split_to_disk) {
+        file.remove(dRepeatScenarios$split_filename)
+      }
+    } else if (parallel & Sys.info()[['sysname']] == 'Windows') {
+      
+      message(paste0('NOTE: Since `parallel=TRUE`, will attempt to use ', parallel_n_cores, ' CPU cores to execute ', nrow(dRepeatScenarios), ' scenarios in parallel.  The user could change the number of parallel cores with the `parallel_n_cores` argument.'))
+
+      cl <- makeCluster(parallel_n_cores)
+      clusterEvalQ(cl, library(dplyr))
+      clusterEvalQ(cl, library(tidyr))
+      clusterEvalQ(cl, library(purrr))
+      if (!parallel_split_to_disk) {
+        clusterExport(cl, varlist=c('dRepeatScenarios', 'data', ls('package:DisImpact')), envir=environment())
+      } else {
+        clusterExport(cl, varlist=c('dRepeatScenarios', ls('package:DisImpact')), envir=environment())
+      }
+
+      dRepeatScenarios$df_results <- parLapply(cl, 1:nrow(dRepeatScenarios)
+                                       , fun=function(i) {
+                                         # data <- data %>% slice(dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist)
+                                         # subset_idx <- dRepeatScenarios[i, ] %>% select(want_indices) %>% unlist
+                                         subset_idx <- dRepeatScenarios %>% slice(i) %>% select(want_indices) %>% unlist
+                                         pmap(dScenarios %>% mutate(subset_idx=list(subset_idx), parallel_split_to_disk=parallel_split_to_disk, split_filename=dRepeatScenarios$split_filename[i]), iterate) %>%
+                                           bind_rows
+                                       }
+                                       )
+      
+      stopCluster(cl=cl)
+      if (parallel_split_to_disk) {
+        file.remove(dRepeatScenarios$split_filename)
+      }
+    }
+    
     # CRAN: no visible binding for global variable
-    df_results <- NULL
+    df_results <- split_filename <- NULL
     
     dRepeatScenarios %>%
-      select(-want_indices) %>%
+      select(-want_indices, -parallel_split_to_disk, -split_filename) %>%
       unnest(df_results)
   }
 }
+
